import socket
import struct

class Connection(object):
    def __init__(self, host, port):
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect((host, port))
    def prep(self):
        self.buf = bytearray(1000000)
        self.bufix = 0
        self.put_int32(0)
    def put_int16(self, v):
        struct.pack_into('<h', self.buf, self.bufix, v)
        self.bufix = self.bufix + 2
    def put_int32(self, v):
        struct.pack_into('<l', self.buf, self.bufix, v)
        self.bufix = self.bufix + 4
    def put_int64(self, v):
        struct.pack_into('<q', self.buf, self.bufix, v)
        self.bufix = self.bufix + 8
    def put_float(self, v):
        struct.pack_into('<f', self.buf, self.bufix, v)
        self.bufix = self.bufix + 4
    def put_double(self, v):
        struct.pack_into('<d', self.buf, self.bufix, v)
        self.bufix = self.bufix + 8
    def put_string(self, v):
        temp = bytes(v, 'iso-8859-1')
        templen = len(temp)
        self.put_int16(templen)
        struct.pack_into(str(templen) + 's', self.buf, self.bufix, temp)
        self.bufix = self.bufix + templen
    def put_block(self, v):
        temp = v.pack()
        templen = len(temp)
        self.put_int16(templen)
        struct.pack_into(str(templen) + 's', self.buf, self.bufix, temp)
        self.bufix = self.bufix + templen
    def send(self):
        wrlen = self.bufix
        struct.pack_into('<l', self.buf, 0, wrlen)
        self.s.sendall(self.buf[0:4+wrlen])
    def recv(self):
        temp = self.s.recv(4)
        rdlen = struct.unpack('<l', temp)[0]
        self.buf = self.s.recv(rdlen)
        self.bufix = 0
    def get_int16(self):
        v = struct.unpack('<h', self.buf[self.bufix:self.bufix+2])[0]
        self.bufix = self.bufix + 2
        return v
    def get_int32(self):
        v = struct.unpack('<l', self.buf[self.bufix:self.bufix+4])[0]
        self.bufix = self.bufix + 4
        return v
    def get_int64(self):
        v = struct.unpack('<q', self.buf[self.bufix:self.bufix+4])[0]
        self.bufix = self.bufix + 8
        return v
    def get_float(self):
        v = struct.unpack('<f', self.buf[self.bufix:self.bufix+4])[0]
        self.bufix = self.bufix + 4
        return v
    def get_double(self):
        v = struct.unpack('<d', self.buf[self.bufix:self.bufix+4])[0]
        self.bufix = self.bufix + 8
        return v
    def get_string(self):
        vlen = self.get_int16()
        v = self.buf[self.bufix:self.bufix+vlen]
        self.bufix = self.bufix + vlen
        return str(v, 'iso-8859-1').rstrip(' \0')
    def get_block(self, o):
        vlen = self.get_int16()
        v = self.buf[self.bufix:self.bufix+vlen]
        o.unpack(v)
    def close(self):
        self.s.close()
        